@@ -121,7 +121,9 @@ class Annotation:
         if rule:
             self.classes = list(set(self.classes) & set(rule.classes))
             self.include_masks(rule)
+            self.merge_include_coords(rule)
             self.exclude_masks(rule)
+            self.exclude_coords(rule)
         if foreground:
             self.make_foreground_mask(
                 slide, size, method=foreground, min_=min_, max_=max_)
@@ -174,6 +176,18 @@ class Annotation:
                             self.masks[cls], self.masks[include])
         self.masks = self.masks_include
 
+    def merge_include_coords(self, rule):
+        """Merge coordinations following the rule.
+
+        Args:
+            rule (wsiprocess.rule.Rule): Rule object.
+        """
+        for cls in self.classes:
+            if hasattr(rule, cls):
+                for include in getattr(rule, cls)["includes"]:
+                    if include in self.mask_coords.keys():
+                        self.mask_coords[cls].extend(self.mask_coords[include])
+
     def exclude_masks(self, rule):
         """Exclude area from base mask with following the rule.
 
@@ -191,6 +205,20 @@ class Annotation:
                             self.masks[cls], overlap_area)
         self.masks = self.masks_exclude
 
+    def exclude_coords(self, rule):
+        """Exclude coordinations following the rule.
+
+        Args:
+            rule (wsiprocess.rule.Rule): Rule object.
+        """
+        for cls in self.classes:
+            if hasattr(rule, cls):
+                base_set = [tuple(c) for c in self.mask_coords[cls]]
+                for exclude in getattr(rule, cls)["excludes"]:
+                    exclude_set = [tuple(c) for c in self.mask_coords[exclude]]
+                    base_set -= exclude_set
+                self.mask_coords[cls] = [list(c) for c in base_set]
+
     def make_foreground_mask(
             self, slide, size=2000, method="otsu", min_=30, max_=190):
         """Make foreground mask.
@@ -209,6 +237,9 @@ class Annotation:
             max (int, optional): Used if method is "minmax". Annotation object
                 defines foreground as the pixels with the value between "min"
                 and "max".
+        
+        TODO:
+            Fix method arg.
         """
         if "foreground" in self.classes:
             return
@@ -220,6 +251,8 @@ class Annotation:
         thumb_gray = cv2.cvtColor(thumb, cv2.COLOR_RGB2GRAY)
         if method == "minmax":
             mask = self._minmax_mask(thumb_gray, min_, max_)
+        elif not isinstance(method, str):
+            mask = method(thumb_gray)
         else:
             mask = self._otsu_method_mask(thumb_gray)
         self.masks["foreground"] = cv2.resize(mask, (slide.width,
