@@ -194,6 +194,25 @@ class Patcher:
             raise NotImplementedError
 
     def find_bbs(self, x, y, cls):
+        """Find bounding boxes which are on the patch.
+
+        Bounding boxes with one of its corners on the patch is on the patch.
+            ex : annotation.mask_coords["benign"][0]
+             = [small_x, small_y, large_x, large_y]
+             = [bbleft, bbtop, bbright, bbbottom]
+
+        TODO:
+            This function can not handle like the case that the annotation is
+            covering the whole area of the patch or the case that one of the
+            side of annotation is striding over the patch.
+
+        Args:
+            x (int): X-axis offset of patch.
+            y (int): Y-axis offset of patch.
+            cls (str): Class of the patch or the bounding box or the segmented
+                area.
+
+        """
         if not self.patch_on_annotation(cls, x, y):
             return []
         else:
@@ -206,48 +225,14 @@ class Patcher:
                     coords[idx] = self.to_bb(coord)
             coords = np.array(coords)
 
-            bblefts = np.min(coords, axis=1)[:, 0]
-            bbtops = np.min(coords, axis=1)[:, 1]
-            bbrights = np.max(coords, axis=1)[:, 0]
-            bbbottoms = np.max(coords, axis=1)[:, 1]
+            idx_of_bb_on_patch = self.corner_on_patch(coords, x, y)
+            idx_of_bb_on_patch += self.side_on_patch(coords, x, y)
+            idx_of_bb_on_patch += self.annotation_cover_patch(coords, x, y)
 
-            # ex : annotation.mask_coords["benign"][0]
-            #  = [small_x, small_y, large_x, large_y]
-            #  = [bbleft, bbtop, bbright, bbbottom]
-            # Bounding boxes with one of its corners on the patch is on the patch.
+            idx_of_bb_on_patch = list(set(idx_of_bb_on_patch))
+            assert idx_of_bb_on_patch, f"[x={x}, y={y}] This patch has annotated area."
 
-            patch_left = x
-            patch_right = x + self.p_width
-            patch_top = y
-            patch_bottom = y + self.p_height
-
-            bbleft_right_of_patch_left = set(
-                np.where(bblefts >= patch_left)[0])
-            bbleft_left_of_patch_right = set(
-                np.where(bblefts <= patch_right)[0])
-            bbright_right_of_patch_left = set(
-                np.where(bbrights >= patch_left)[0])
-            bbright_left_of_patch_right = set(
-                np.where(bbrights <= patch_right)[0])
-            bbtop_below_patch_top = set(np.where(bbtops >= patch_top)[0])
-            bbtop_above_patch_bottom = set(np.where(bbtops <= patch_bottom)[0])
-            bbbottom_below_patch_top = set(np.where(bbbottoms >= patch_top)[0])
-            bbbottom_above_patch_bottom = set(
-                np.where(bbbottoms <= patch_bottom)[0])
-
-            bbleft_on_patch = bbleft_right_of_patch_left & bbleft_left_of_patch_right
-            bbright_on_patch = bbright_right_of_patch_left & bbright_left_of_patch_right
-            bbtop_on_patch = bbtop_above_patch_bottom & bbtop_below_patch_top
-            bbbottom_on_patch = bbbottom_above_patch_bottom & bbbottom_below_patch_top
-
-            bb_lefttop_on_patch = bbleft_on_patch & bbtop_on_patch
-            bb_leftbottom_on_patch = bbleft_on_patch & bbbottom_on_patch
-            bb_righttop_on_patch = bbright_on_patch & bbtop_on_patch
-            bb_rightbottom_on_patch = bbright_on_patch & bbbottom_on_patch
-
-            idx_of_bb_on_patch = bb_lefttop_on_patch | bb_leftbottom_on_patch | bb_righttop_on_patch | bb_rightbottom_on_patch
-
-            bbs_raw = coords[list(idx_of_bb_on_patch)]
+            bbs_raw = coords[idx_of_bb_on_patch]
             bbs = []
             for bb_raw in bbs_raw:
                 x1 = bb_raw[:, 0].min()
@@ -266,6 +251,130 @@ class Patcher:
                 bbs.append(bb)
             return bbs
 
+    def corner_on_patch(self, coords, x, y):
+        """Check if at least one of the corners is on the patch.
+
+        Args:
+            coords (np.array) : Coordinations of annotations.
+            px (int) : X coordinate of left top corner of the patch.
+            py (int) : Y coordinate of left top corner of the patch.
+
+        Returns:
+            idx_of_bb_on_patch (list): List of np.int64s which are the indices
+                of bounding boxes on the patch.
+        """
+        bblefts = np.min(coords, axis=1)[:, 0]
+        bbtops = np.min(coords, axis=1)[:, 1]
+        bbrights = np.max(coords, axis=1)[:, 0]
+        bbbottoms = np.max(coords, axis=1)[:, 1]
+
+        patch_left = x
+        patch_right = x + self.p_width
+        patch_top = y
+        patch_bottom = y + self.p_height
+
+        left_on_patch = (patch_left <= bblefts) & (bblefts <= patch_right)
+        right_on_patch = (patch_left <= bbrights) & (bbrights <= patch_right)
+        top_on_patch = (patch_top <= bbtops) & (bbtops <= patch_bottom)
+        bottom_on_patch = (patch_top <= bbbottoms) & (bbbottoms <= patch_bottom)
+
+        topleft_on_patch = left_on_patch & top_on_patch
+        topright_on_patch = right_on_patch & top_on_patch
+        bottomleft_on_patch = left_on_patch & bottom_on_patch
+        bottomright_on_patch = right_on_patch & bottom_on_patch
+
+        idx_of_bb_on_patch = topleft_on_patch | topright_on_patch |\
+            bottomleft_on_patch | bottomright_on_patch
+        idx_of_bb_on_patch = np.where(idx_of_bb_on_patch)[0]
+        return list(idx_of_bb_on_patch)
+
+    def side_on_patch(self, coords, x, y):
+        """Check if at least one of the side is on the patch.
+
+        Args:
+            coords (np.array) : Coordinations of annotations.
+            px (int) : X coordinate of left top corner of the patch.
+            py (int) : Y coordinate of left top corner of the patch.
+
+        Returns:
+            idx_of_bb_on_patch (list): List of np.int64s which are the indices
+                of bounding boxes on the patch.
+        """
+        bblefts = np.min(coords, axis=1)[:, 0]
+        bbtops = np.min(coords, axis=1)[:, 1]
+        bbrights = np.max(coords, axis=1)[:, 0]
+        bbbottoms = np.max(coords, axis=1)[:, 1]
+
+        patch_left = x
+        patch_right = x + self.p_width
+        patch_top = y
+        patch_bottom = y + self.p_height
+
+        left_on_patch = (patch_left <= bblefts) & (bblefts <= patch_right)
+        right_on_patch = (patch_left <= bbrights) & (bbrights <= patch_right)
+        top_on_patch = (patch_top <= bbtops) & (bbtops <= patch_bottom)
+        bottom_on_patch = (patch_top <= bbbottoms) & (bbbottoms <= patch_bottom)
+
+        # One side is on the patch
+        leftside_on_patch = (left_on_patch) &\
+                            (bbtops <= patch_top) &\
+                            (patch_bottom <= bbbottoms)
+        rightside_on_patch = (right_on_patch) &\
+                             (bbtops <= patch_top) &\
+                             (patch_bottom <= bbbottoms)
+        topside_on_patch = (bblefts <= patch_left) &\
+                           (patch_right <= bbrights) &\
+                           (top_on_patch)
+        bottomside_on_patch = (bblefts <= patch_left) &\
+                              (patch_right <= bbrights) &\
+                              (bottom_on_patch)
+
+        # Two sides are on the patch
+        leftrightside_on_patch = (left_on_patch) &\
+                                 (right_on_patch) &\
+                                 (bbtops <= patch_top) &\
+                                 (patch_bottom <= bbbottoms)
+        topbottomside_on_patch = (bblefts <= patch_left) &\
+                                 (patch_right <= bbrights) &\
+                                 (top_on_patch) &\
+                                 (bottom_on_patch)
+
+        idx_of_bb_on_patch = leftside_on_patch | rightside_on_patch | \
+            topside_on_patch | bottomside_on_patch | \
+            leftrightside_on_patch | topbottomside_on_patch
+        idx_of_bb_on_patch = np.where(idx_of_bb_on_patch)[0]
+        idx_of_bb_on_patch = list(set(idx_of_bb_on_patch))
+        return list(idx_of_bb_on_patch)
+
+    def annotation_cover_patch(self, coords, x, y):
+        """Check if the annotation is covering the whole patch.
+
+        Args:
+            coords (np.array) : Coordinations of annotations.
+            px (int) : X coordinate of left top corner of the patch.
+            py (int) : Y coordinate of left top corner of the patch.
+
+        Returns:
+            idx_of_bb_on_patch (list): List of np.int64s which are the indices
+                of bounding boxes on the patch.
+        """
+        bblefts = np.min(coords, axis=1)[:, 0]
+        bbtops = np.min(coords, axis=1)[:, 1]
+        bbrights = np.max(coords, axis=1)[:, 0]
+        bbbottoms = np.max(coords, axis=1)[:, 1]
+
+        patch_left = x
+        patch_right = x + self.p_width
+        patch_top = y
+        patch_bottom = y + self.p_height
+
+        idx_of_bb_on_patch = (bblefts <= patch_left) &\
+                             (patch_right <= bbrights) &\
+                             (bbtops <= patch_top) &\
+                             (patch_bottom <= bbbottoms)
+        idx_of_bb_on_patch = np.where(idx_of_bb_on_patch)[0]
+        return list(idx_of_bb_on_patch)
+
     def to_bb(self, coord):
         """Convert coordinates to voc coordinates.
 
@@ -320,6 +429,14 @@ class Patcher:
             masks.append(mask)
             return masks
 
+    def remove_dup_in_results(self):
+        """Remove duplicate results in self.result["results"]
+        """
+        results = set()
+        for result in self.result["result"]:
+            results.add(json.dumps(result))
+        self.result["result"] = [json.loads(res) for res in list(results)]
+
     def save_results(self):
         """Save the extraction results.
 
@@ -344,6 +461,8 @@ class Patcher:
         self.result["save_to"] = str(Path(self.save_to).absolute())
         self.result["classes"] = sorted(self.classes)
 
+        self.remove_dup_in_results()
+
         with open("{}/{}/results.json".format(self.save_to, self.filestem), "w") as f:
             json.dump(self.result, f, indent=4)
 
